@@ -120,10 +120,13 @@ class MeasureFrame(InvincibleFrame):
 	exitcode = 1
 
 	def __init__(self, parent=None, id=-1):
+		style = wx.DEFAULT_FRAME_STYLE & ~(wx.RESIZE_BORDER | wx.MAXIMIZE_BOX)
+		if (os.getenv("XDG_SESSION_TYPE") != "wayland" and
+			getcfg("patterngenerator.use_pattern_window")):
+			style |= wx.STAY_ON_TOP
 		InvincibleFrame.__init__(self, parent, id, 
 								 lang.getstr("measureframe.title"), 
-								 style=wx.DEFAULT_FRAME_STYLE & 
-									   ~(wx.RESIZE_BORDER | wx.MAXIMIZE_BOX),
+								 style=style,
 								 name="measureframe")
 		self.SetIcons(config.get_icon_bundle([256, 48, 32, 16], appname))
 		self.Bind(wx.EVT_CLOSE, self.close_handler, self)
@@ -376,6 +379,7 @@ class MeasureFrame(InvincibleFrame):
 		if debug: safe_print("[D]  measureframe_size:", size)
 		if (sys.platform not in ("darwin", "win32") and
 			os.getenv("XDG_SESSION_TYPE") != "wayland" and
+			not os.getenv("XDG_CURRENT_DESKTOP", "").startswith("KDE") and
 			hasattr(self, "MaxClientSize")):
 			self.MaxClientSize = (-1, -1)
 			self.MinClientSize = size
